import { NextRequest } from 'next/server';
import { JWTVerifyResult } from 'jose';

interface Provider {
  handleCallback(request: NextRequest): Promise<Response>;
  handleSignIn(request: NextRequest, referer?: string): Promise<Response>;
  handleSignOut(request: NextRequest): Promise<void>;
  handleAuthCheck(token: string): Promise<{ user: UserType } | false>;
}

interface Database {
  createAccount(data: AccountType): Promise<AccountType>;
  createUser(data: UserType): Promise<UserType>;
  createSession(data: SessionType): Promise<SessionType>;
  findAccount(data: Partial<AccountType>): Promise<AccountType | undefined>;
  findUser(data: Partial<UserType>): Promise<UserType | undefined>;
  findSession(data: Partial<SessionType>): Promise<SessionType | undefined>;
  purgeSessions(user: UserType) : Promise<boolean>;
}

interface Cache {
  getValue(key: string): Promise<string | null>;
  setValue(key: string, value: string, options?: { expire: number; }): Promise<unknown>;
  deleteKey(key: string): Promise<number>;
}

interface SingleConfig {
  provider: Provider,
  database: Database,
  cache: Cache
}

interface Config {
  [key: string]: SingleConfig
}

type Secret = string

interface Session {
  user: UserType;
}

interface AccountType {
  accountId: string;
  provider: string;
  accessToken: string;
  refreshToken: string;
  expiresAt?: Date | undefined;
  tokenType?: string;
  scope: string;
}

interface SessionType {
  sessionToken: string;
  provider: string
  accountId: string
  expiresAt: Date
}

interface UserType {
  name: string;
  email: string;
  image: string;
  accountId: string;
  provider: string;
  emailVerified: boolean;
}

type ErrorResult = { success: false, message: string, status: number }
type JsonResult<T> = { success: true, data: T }

interface SignInConfig {
  redirect_url?: string
  email?: string,
  password?: string,
  method?: string,
}
type SignInWithProviderResult = Promise<false | JsonResult<{ url: string }> | ErrorResult>

declare module './client' {
  export const signIn: typeof signIn;
  export const signOut: typeof signOut;
}

declare function sessionsInternal(): Promise<false | Session>;

/**
 * Sends an error response with the given status and message.
 *
 * @param {number} status - The HTTP status code of the error response.
 * @param {string} message - The error message to be included in the response.
 * @returns {object} - The error response object.
 */
declare const sendError: (status: number, message: string) => Response;
/**
 * Creates a JSON response object with optional status code.
 *
 * @param {any} data - The data to be included in the response.
 * @param {number} [code] - The optional status code for the response.
 * @returns {object} - The JSON response object.
 */
declare const sendJson: (data: any, code?: number) => Response;
/**
 * Redirects to the error page with the provided status and message.
 *
 * @param {number} status - The HTTP status code of the error.
 * @param {string} message - The error message.
 */
declare const sendErrorRedirect: (status: number, message: string) => Response;

interface CreatTokenPayload {
    provider: string;
    email: string;
    sub: string;
    [key: string]: string;
}
declare const createToken: (data: CreatTokenPayload, expiration: Date) => Promise<string>;
declare const decodeToken: <T = CreatTokenPayload>(sessionTokenString: string) => Promise<false | JWTVerifyResult<T>>;

declare class Auth {
    static config: Config;
    static secret: Secret;
    static routes: any;
    constructor(config: Config, secret: any);
}
declare const auth: typeof sessionsInternal;

export { Auth as A, type SignInConfig as S, type SignInWithProviderResult as a, sendJson as b, sendErrorRedirect as c, decodeToken as d, createToken as e, auth as f, sendError as s };
